/*
 * Copyright 1989 - 1992, John F. Haugh II
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by John F. Haugh, II
 *      and other contributors.
 * 4. Neither the name of John F. Haugh, II nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY JOHN HAUGH AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL JOHN HAUGH OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <config.h>

#include "rcsid.h"
RCSID("$Id: env.c,v 1.1.1.1 1996/08/10 07:59:50 marekm Exp $")

#include <stdio.h>
#include "defines.h"

extern	char	**environ;
extern	char	*newenvp[];
extern	int	newenvc;
extern	int	maxenv;

char *xmalloc();
char *xstrdup();
void free();

static char * const forbid[] = {
	"ENV=",
	"HOME=",
	"IFS=",
	"LD_",
	"PATH=",
	"SHELL=",
	(char *) 0
};

/*
 * addenv - add a new environmental entry
 */

void
addenv(entry)
	const char *entry;
{
	char	*cp;
	int	i;
	int	len;

	if ((cp = strchr (entry, '=')))
		len = cp - entry;
	else
		return;

	for (i = 0;i < newenvc;i++)
		if (strncmp (entry, newenvp[i], len) == 0 &&
			(newenvp[i][len] == '=' || newenvp[i][len] == '\0'))
			break;

	if (i == maxenv) {
		puts ("Environment overflow");
		return;
	}
	if (i == newenvc) {
		newenvp[newenvc++] = xstrdup (entry);
	} else {
		free (newenvp[i]);
		newenvp[i] = xstrdup (entry);
	}
}

void
addenv2(name, value)
	const char *name, *value;
{
	char *tmp = xmalloc(strlen(name) + strlen(value) + 2);

	sprintf(tmp, "%s=%s", name, value);
	addenv(tmp);
	free(tmp);
}

/*
 * set_env - copy command line arguments into the environment
 */

void
set_env (argc, argv)
	int argc;
	char * const *argv;
{
	int	noname = 1;
	char	variable[BUFSIZ];
	char	*cp;

	for ( ; argc > 0; argc--, argv++) {
		if (strlen(*argv) >= sizeof variable)
			continue;	/* ignore long entries */

		if (! (cp = strchr (*argv, '='))) {
			sprintf(variable, "L%d", noname++);
			addenv2(variable, *argv);
		} else {
			char * const *p;

			for (p = forbid; *p; p++)
				if (strncmp(*argv, *p, strlen(*p)) == 0)
					break;

			if (*p) {
				strncpy(variable, *argv, cp - *argv);
				variable[cp - *argv] = '\0';
				printf("You may not change $%s\n", variable);
				continue;
			}

			addenv(*argv);
		}
	}
}
